@@ -186,7 +186,7 @@ connect_done (GObject      *source_object,
 	BluetoothSettingsWidget *self;
 	BluetoothSettingsWidgetPrivate *priv;
 	gboolean success;
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 	ConnectData *data = (ConnectData *) user_data;
 
 	success = bluetooth_client_connect_service_finish (BLUETOOTH_CLIENT (source_object),
@@ -212,7 +212,6 @@ connect_done (GObject      *source_object,
 		set_connecting_page (self, CONNECTING_NOTEBOOK_PAGE_SWITCH);
 	}
 
-	g_clear_error (&error);
 	remove_connecting (self, data->bdaddr);
 
 	//FIXME show an error if it failed?
@@ -221,7 +220,6 @@ connect_done (GObject      *source_object,
 		      NULL);
 
 out:
-	g_clear_error (&error);
 	g_free (data->bdaddr);
 	g_free (data);
 }
@@ -262,8 +260,8 @@ get_properties_for_device (BluetoothSettingsWidget  *self,
 			   BluetoothType            *type)
 {
 	BluetoothSettingsWidgetPrivate *priv = BLUETOOTH_SETTINGS_WIDGET_GET_PRIVATE (self);
-	GVariant *value;
-	char *bdaddr;
+	g_autoptr(GVariant) value = NULL;
+	g_autofree char *bdaddr = NULL;
 
 	g_return_val_if_fail (name != NULL, FALSE);
 
@@ -273,11 +271,11 @@ get_properties_for_device (BluetoothSettingsWidget  *self,
 		return FALSE;
 	}
 	*name = g_variant_dup_string (value, NULL);
-	g_variant_unref (value);
+	g_clear_pointer (&value, g_variant_unref);
 
 	value = g_dbus_proxy_get_cached_property (device, "Address");
 	bdaddr = g_variant_dup_string (value, NULL);
-	g_variant_unref (value);
+	g_clear_pointer (&value, g_variant_unref);
 
 	if (ret_bdaddr)
 		*ret_bdaddr = g_strdup (bdaddr);
@@ -286,7 +284,6 @@ get_properties_for_device (BluetoothSettingsWidget  *self,
 		value = g_dbus_proxy_get_cached_property (device, "Class");
 		if (value != NULL) {
 			*type = bluetooth_class_to_type (g_variant_get_uint32 (value));
-			g_variant_unref (value);
 		} else {
 			*type = GPOINTER_TO_UINT (g_hash_table_lookup (priv->devices_type, bdaddr));
 			if (*type == 0)
@@ -294,8 +291,6 @@ get_properties_for_device (BluetoothSettingsWidget  *self,
 		}
 	}
 
-	g_free (bdaddr);
-
 	return TRUE;
 }
 
@@ -326,13 +321,12 @@ get_icade_pincode (char **pin_display_str)
 
 	for (i = 0; i < PIN_NUM_DIGITS; i++) {
 		int r;
-		char *c;
+		g_autofree char *c = NULL;
 
 		r = g_random_int_range (1, 5);
 
 		c = g_strdup_printf ("%d", r);
 		g_string_append (pin, c);
-		g_free (c);
 
 		g_string_append (pin_display, arrows[r]);
 	}
@@ -364,7 +358,7 @@ enter_pin_cb (GtkDialog *dialog,
 
 	if (response == GTK_RESPONSE_ACCEPT) {
 		const char *name;
-		char *pin;
+		g_autofree char *pin = NULL;
 		BluetoothPairingMode mode;
 
 		mode = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (dialog), "mode"));
@@ -379,7 +373,6 @@ enter_pin_cb (GtkDialog *dialog,
 		}
 		bluetooth_pairing_dialog_set_mode (BLUETOOTH_PAIRING_DIALOG (priv->pairing_dialog),
 						   mode, pin, name);
-		g_free (pin);
 		g_signal_connect (G_OBJECT (priv->pairing_dialog), "response",
 				  G_CALLBACK (display_cb), user_data);
 	} else {
@@ -431,22 +424,22 @@ pincode_callback (GDBusMethodInvocation *invocation,
 	BluetoothSettingsWidget *self = user_data;
 	BluetoothSettingsWidgetPrivate *priv = BLUETOOTH_SETTINGS_WIDGET_GET_PRIVATE (user_data);
 	BluetoothType type;
-	char *name, *bdaddr;
+	g_autofree char *name = NULL;
+	g_autofree char *bdaddr = NULL;
 	guint max_digits;
 	gboolean confirm_pin = TRUE;
-	char *default_pin;
-	char *display_pin = NULL;
+	g_autofree char *default_pin = NULL;
+	g_autofree char *display_pin = NULL;
 	BluetoothPairingMode mode;
 	gboolean remote_initiated;
 
 	g_debug ("pincode_callback (%s)", g_dbus_proxy_get_object_path (device));
 
 	if (!get_properties_for_device (self, device, &name, &bdaddr, &type)) {
-		char *msg;
+		g_autofree char *msg = NULL;
 
 		msg = g_strdup_printf ("Missing information for %s", g_dbus_proxy_get_object_path (device));
 		g_dbus_method_invocation_return_dbus_error (invocation, "org.bluez.Error.Rejected", msg);
-		g_free (msg);
 		return;
 	}
 
@@ -507,7 +500,7 @@ pincode_callback (GDBusMethodInvocation *invocation,
 						   BLUETOOTH_PAIRING_MODE_YES_NO,
 						   default_pin, name);
 
-		g_object_set_data_full (G_OBJECT (invocation), "pin", g_strdup (default_pin), g_free);
+		g_object_set_data_full (G_OBJECT (invocation), "pin", g_steal_pointer (&default_pin), g_free);
 		g_object_set_data_full (G_OBJECT (invocation), "device", g_object_ref (device), g_object_unref);
 		g_object_set_data (G_OBJECT (priv->pairing_dialog), "invocation", invocation);
 
@@ -515,14 +508,50 @@ pincode_callback (GDBusMethodInvocation *invocation,
 				  G_CALLBACK (confirm_remote_pin_cb), user_data);
 	}
 
-	g_free (name);
-	g_free (bdaddr);
-	g_free (default_pin);
-	g_free (display_pin);
-
 	gtk_widget_show (priv->pairing_dialog);
 }
 
+static void
+cancel_setup_cb (GObject      *source_object,
+		 GAsyncResult *res,
+		 gpointer      user_data)
+{
+	g_autoptr(GError) error = NULL;
+	g_autofree char *path = NULL;
+	gboolean ret;
+
+	ret = bluetooth_client_cancel_setup_device_finish (BLUETOOTH_CLIENT (source_object),
+							   res, &path, &error);
+	if (!ret)
+		g_debug ("Setup cancellation for '%s' failed: %s", path, error->message);
+	else
+		g_debug ("Setup cancellation for '%s' success", path);
+}
+
+static void
+display_passkey_or_pincode_cb (GtkDialog *dialog,
+			       int        response,
+			       gpointer   user_data)
+{
+	BluetoothSettingsWidgetPrivate *priv = BLUETOOTH_SETTINGS_WIDGET_GET_PRIVATE (user_data);
+
+	if (response == GTK_RESPONSE_CANCEL ||
+	    response == GTK_RESPONSE_DELETE_EVENT) {
+		g_autofree char *path = NULL;
+
+		path = g_object_steal_data (G_OBJECT (dialog), "path");
+		bluetooth_client_cancel_setup_device (priv->client,
+						      path,
+						      priv->cancellable,
+						      cancel_setup_cb,
+						      user_data);
+	} else {
+		g_assert_not_reached ();
+	}
+
+	g_clear_pointer (&priv->pairing_dialog, gtk_widget_destroy);
+}
+
 static void
 display_callback (GDBusMethodInvocation *invocation,
 		  GDBusProxy            *device,
@@ -532,7 +561,8 @@ display_callback (GDBusMethodInvocation *invocation,
 {
 	BluetoothSettingsWidget *self = user_data;
 	BluetoothSettingsWidgetPrivate *priv = BLUETOOTH_SETTINGS_WIDGET_GET_PRIVATE (user_data);
-	char *pin_str, *name;
+	g_autofree char *pin_str = NULL;
+	g_autofree char *name = NULL;
 
 	g_debug ("display_callback (%s, %i, %i)", g_dbus_proxy_get_object_path (device), pin, entered);
 
@@ -548,8 +578,11 @@ display_callback (GDBusMethodInvocation *invocation,
 					   name);
 	bluetooth_pairing_dialog_set_pin_entered (BLUETOOTH_PAIRING_DIALOG (priv->pairing_dialog),
 						  entered);
-	g_free (pin_str);
-	g_free (name);
+	g_signal_connect (G_OBJECT (priv->pairing_dialog), "response",
+			  G_CALLBACK (display_passkey_or_pincode_cb), user_data);
+	g_object_set_data_full (G_OBJECT (priv->pairing_dialog),
+				"path", g_strdup (g_dbus_proxy_get_object_path (device)),
+				g_free);
 
 	gtk_widget_show (priv->pairing_dialog);
 }
@@ -563,18 +596,18 @@ display_pincode_callback (GDBusMethodInvocation *invocation,
 	BluetoothSettingsWidget *self = user_data;
 	BluetoothSettingsWidgetPrivate *priv = BLUETOOTH_SETTINGS_WIDGET_GET_PRIVATE (user_data);
 	BluetoothType type;
-	char *display_pin = NULL;
-	char *name, *bdaddr;
-	char *db_pin;
+	g_autofree char *display_pin = NULL;
+	g_autofree char *name = NULL;
+	g_autofree char *bdaddr = NULL;
+	g_autofree char *db_pin = NULL;
 
 	g_debug ("display_pincode_callback (%s, %s)", g_dbus_proxy_get_object_path (device), pincode);
 
 	if (!get_properties_for_device (self, device, &name, &bdaddr, &type)) {
-		char *msg;
+		g_autofree char *msg = NULL;
 
 		msg = g_strdup_printf ("Missing information for %s", g_dbus_proxy_get_object_path (device));
 		g_dbus_method_invocation_return_dbus_error (invocation, "org.bluez.Error.Rejected", msg);
-		g_free (msg);
 		return;
 	}
 
@@ -587,23 +620,21 @@ display_pincode_callback (GDBusMethodInvocation *invocation,
 	if (g_strcmp0 (db_pin, "KEYBOARD") == 0) {
 		/* Should work, follow through */
 	} else if (g_strcmp0 (db_pin, "ICADE") == 0) {
-		char *msg;
+		g_autofree char *msg = NULL;
 
 		msg = g_strdup_printf ("Generated pincode for %s when it shouldn't have", name);
 		g_dbus_method_invocation_return_dbus_error (invocation, "org.bluez.Error.Rejected", msg);
-		g_free (msg);
-		goto bail;
+		return;
 	} else if (g_strcmp0 (db_pin, "0000") == 0) {
 		g_debug ("Ignoring generated keyboard PIN '%s', should get 0000 soon", pincode);
 		g_dbus_method_invocation_return_value (invocation, NULL);
-		goto bail;
+		return;
 	} else if (g_strcmp0 (db_pin, "NULL") == 0) {
-		char *msg;
+		g_autofree char *msg = NULL;
 
 		msg = g_strdup_printf ("Attempting pairing for %s that doesn't support pairing", name);
 		g_dbus_method_invocation_return_dbus_error (invocation, "org.bluez.Error.Rejected", msg);
-		g_free (msg);
-		goto bail;
+		return;
 	}
 
 	setup_pairing_dialog (BLUETOOTH_SETTINGS_WIDGET (user_data));
@@ -613,15 +644,14 @@ display_pincode_callback (GDBusMethodInvocation *invocation,
 					   BLUETOOTH_PAIRING_MODE_PIN_DISPLAY_KEYBOARD,
 					   display_pin,
 					   name);
+	g_signal_connect (G_OBJECT (priv->pairing_dialog), "response",
+			  G_CALLBACK (display_passkey_or_pincode_cb), user_data);
+	g_object_set_data_full (G_OBJECT (priv->pairing_dialog),
+				"path", g_strdup (g_dbus_proxy_get_object_path (device)),
+				g_free);
 	gtk_widget_show (priv->pairing_dialog);
 
 	g_dbus_method_invocation_return_value (invocation, NULL);
-
-bail:
-	g_free (db_pin);
-	g_free (display_pin);
-	g_free (bdaddr);
-	g_free (name);
 }
 
 static void
@@ -683,7 +713,8 @@ confirm_callback (GDBusMethodInvocation *invocation,
 {
 	BluetoothSettingsWidget *self = user_data;
 	BluetoothSettingsWidgetPrivate *priv = BLUETOOTH_SETTINGS_WIDGET_GET_PRIVATE (user_data);
-	char *name, *pin_str;
+	g_autofree char *name = NULL;
+	g_autofree char *pin_str = NULL;
 
 	g_debug ("confirm_callback (%s, %i)", g_dbus_proxy_get_object_path (device), pin);
 
@@ -700,9 +731,6 @@ confirm_callback (GDBusMethodInvocation *invocation,
 	g_object_set_data (G_OBJECT (priv->pairing_dialog), "invocation", invocation);
 
 	gtk_widget_show (priv->pairing_dialog);
-
-	g_free (pin_str);
-	g_free (name);
 }
 
 static void
@@ -712,7 +740,7 @@ authorize_callback (GDBusMethodInvocation *invocation,
 {
 	BluetoothSettingsWidget *self = user_data;
 	BluetoothSettingsWidgetPrivate *priv = BLUETOOTH_SETTINGS_WIDGET_GET_PRIVATE (user_data);
-	char *name;
+	g_autofree char *name = NULL;
 
 	g_debug ("authorize_callback (%s)", g_dbus_proxy_get_object_path (device));
 
@@ -727,8 +755,6 @@ authorize_callback (GDBusMethodInvocation *invocation,
 	g_object_set_data (G_OBJECT (priv->pairing_dialog), "invocation", invocation);
 
 	gtk_widget_show (priv->pairing_dialog);
-
-	g_free (name);
 }
 
 static void
@@ -748,11 +774,10 @@ authorize_service_cb (GtkDialog *dialog,
 		bluetooth_client_set_trusted (priv->client, g_dbus_proxy_get_object_path (device), TRUE);
 		g_dbus_method_invocation_return_value (invocation, NULL);
 	} else {
-		char *msg;
+		g_autofree char *msg = NULL;
 
 		msg = g_strdup_printf ("Rejecting service auth (HID): not paired or trusted");
 		g_dbus_method_invocation_return_dbus_error (invocation, "org.bluez.Error.Rejected", msg);
-		g_free (msg);
 	}
 	g_clear_pointer (&priv->pairing_dialog, gtk_widget_destroy);
 }
@@ -765,7 +790,6 @@ authorize_service_callback (GDBusMethodInvocation *invocation,
 {
 	BluetoothSettingsWidget *self = user_data;
 	BluetoothSettingsWidgetPrivate *priv = BLUETOOTH_SETTINGS_WIDGET_GET_PRIVATE (user_data);
-	char *msg;
 	GVariant *value;
 	gboolean paired, trusted;
 
@@ -789,10 +813,10 @@ authorize_service_callback (GDBusMethodInvocation *invocation,
 	}
 
 	if (g_strcmp0 (bluetooth_uuid_to_string (uuid), "HumanInterfaceDeviceService") != 0) {
+		g_autofree char *msg = NULL;
 		msg = g_strdup_printf ("Rejecting service auth (%s) for %s: not HID",
 				       uuid, g_dbus_proxy_get_object_path (device));
 		g_dbus_method_invocation_return_dbus_error (invocation, "org.bluez.Error.Rejected", msg);
-		g_free (msg);
 		return;
 	}
 
@@ -800,7 +824,7 @@ authorize_service_callback (GDBusMethodInvocation *invocation,
 	if (paired || trusted) {
 		g_dbus_method_invocation_return_value (invocation, NULL);
 	} else {
-		char *name;
+		g_autofree char *name = NULL;
 
 		setup_pairing_dialog (BLUETOOTH_SETTINGS_WIDGET (user_data));
 		get_properties_for_device (self, device, &name, NULL, NULL);
@@ -814,8 +838,6 @@ authorize_service_callback (GDBusMethodInvocation *invocation,
 		g_object_set_data (G_OBJECT (priv->pairing_dialog), "invocation", invocation);
 
 		gtk_widget_show (priv->pairing_dialog);
-
-		g_free (name);
 	}
 }
 
@@ -858,7 +880,7 @@ connect_timeout_cb (gpointer user_data)
 	SetupConnectData *data = (SetupConnectData *) user_data;
 	BluetoothSettingsWidgetPrivate *priv = BLUETOOTH_SETTINGS_WIDGET_GET_PRIVATE (data->self);
 
-	bluetooth_client_connect_service (priv->client, data->device, TRUE, NULL, connect_callback, data);
+	bluetooth_client_connect_service (priv->client, data->device, TRUE, priv->cancellable, connect_callback, data);
 	data->timeout_id = 0;
 
 	return G_SOURCE_REMOVE;
@@ -871,22 +893,20 @@ connect_callback (GObject      *source_object,
 {
 	SetupConnectData *data = (SetupConnectData *) user_data;
 	BluetoothSettingsWidgetPrivate *priv;
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 	gboolean success;
 
 	success = bluetooth_client_connect_service_finish (BLUETOOTH_CLIENT (source_object), res, &error);
 
 	if (success == FALSE) {
 		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-			g_error_free (error);
 			goto bail;
 		} else if (g_timer_elapsed (data->timer, NULL) < CONNECT_TIMEOUT) {
 			g_assert (data->timeout_id == 0);
 			data->timeout_id = g_timeout_add (500, connect_timeout_cb, data);
-			g_error_free (error);
 			return;
 		}
-		g_debug ("Failed to connect to device %s", data->device);
+		g_debug ("Failed to connect to device %s: %s", data->device, error->message);
 	}
 
 	turn_off_pairing (data->self, data->device);
@@ -913,9 +933,9 @@ create_callback (GObject      *source_object,
 	BluetoothSettingsWidget *self = user_data;
 	BluetoothSettingsWidgetPrivate *priv = BLUETOOTH_SETTINGS_WIDGET_GET_PRIVATE (self);
 	SetupConnectData *data;
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 	gboolean ret;
-	char *path;
+	g_autofree char *path = NULL;
 
 	ret = bluetooth_client_setup_device_finish (BLUETOOTH_CLIENT (source_object),
 						    res, &path, &error);
@@ -923,13 +943,10 @@ create_callback (GObject      *source_object,
 	/* Create failed */
 	if (ret == FALSE) {
 		//char *text;
-		char *dbus_error;
+		g_autofree char *dbus_error = NULL;
 
-		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-			g_error_free (error);
-			g_free (path);
+		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 			return;
-		}
 
 		priv = BLUETOOTH_SETTINGS_WIDGET_GET_PRIVATE (user_data);
 
@@ -956,10 +973,6 @@ create_callback (GObject      *source_object,
 			//g_free (text);
 		}
 
-		g_free (dbus_error);
-		g_error_free (error);
-		g_free (path);
-
 		g_object_set (G_OBJECT (priv->client),
 			      "default-adapter-discovering", has_default_adapter (self),
 			      NULL);
@@ -976,11 +989,11 @@ create_callback (GObject      *source_object,
 
 	data = g_new0 (SetupConnectData, 1);
 	data->self = user_data;
-	data->device = path;
+	data->device = g_steal_pointer (&path);
 	data->timer = g_timer_new ();
 
 	bluetooth_client_connect_service (BLUETOOTH_CLIENT (source_object),
-					  path, TRUE, priv->cancellable, connect_callback, data);
+					  data->device, TRUE, priv->cancellable, connect_callback, data);
 	//gtk_assistant_set_current_page (window_assistant, PAGE_FINISHING);
 }
 
@@ -992,7 +1005,7 @@ device_name_appeared (GObject    *gobject,
 		      GParamSpec *pspec,
 		      gpointer    user_data)
 {
-	char *name;
+	g_autofree char *name = NULL;
 
 	g_object_get (G_OBJECT (gobject),
 		      "name", &name,
@@ -1002,7 +1015,6 @@ device_name_appeared (GObject    *gobject,
 
 	g_debug ("Pairing device name is now '%s'", name);
 	start_pairing (user_data, GTK_LIST_BOX_ROW (gobject));
-	g_free (name);
 
 	g_signal_handlers_disconnect_by_func (gobject, device_name_appeared, user_data);
 }
@@ -1012,10 +1024,11 @@ start_pairing (BluetoothSettingsWidget *self,
 	       GtkListBoxRow           *row)
 {
 	BluetoothSettingsWidgetPrivate *priv = BLUETOOTH_SETTINGS_WIDGET_GET_PRIVATE (self);
-	GDBusProxy *proxy;
+	g_autoptr(GDBusProxy) proxy = NULL;
 	gboolean pair = TRUE;
 	BluetoothType type;
-	char *bdaddr, *name;
+	g_autofree char *bdaddr = NULL;
+	g_autofree char *name = NULL;
 	gboolean legacy_pairing;
 	const char *pincode;
 
@@ -1032,9 +1045,6 @@ start_pairing (BluetoothSettingsWidget *self,
 		g_debug ("No name yet, will start pairing later");
 		g_signal_connect (G_OBJECT (row), "notify::name",
 				  G_CALLBACK (device_name_appeared), self);
-		g_object_unref (proxy);
-		g_free (bdaddr);
-		g_free (name);
 		return;
 	}
 
@@ -1065,7 +1075,6 @@ start_pairing (BluetoothSettingsWidget *self,
 				       priv->cancellable,
 				       (GAsyncReadyCallback) create_callback,
 				       self);
-	g_object_unref (proxy);
 }
 
 static gboolean
@@ -1114,7 +1123,7 @@ update_properties (BluetoothSettingsWidget *self,
 	gboolean ret;
 	BluetoothType type;
 	gboolean connected, paired;
-	char **uuids, *bdaddr, *name, *icon;
+	char **uuids, *bdaddr, *alias, *icon;
 	guint i;
 
 	model = bluetooth_client_get_device_model (priv->client);
@@ -1144,7 +1153,7 @@ update_properties (BluetoothSettingsWidget *self,
 
 	gtk_tree_model_get (model, &iter,
 			    BLUETOOTH_COLUMN_ADDRESS, &bdaddr,
-			    BLUETOOTH_COLUMN_NAME, &name,
+			    BLUETOOTH_COLUMN_ALIAS, &alias,
 			    BLUETOOTH_COLUMN_ICON, &icon,
 			    BLUETOOTH_COLUMN_PAIRED, &paired,
 			    BLUETOOTH_COLUMN_CONNECTED, &connected,
@@ -1165,9 +1174,9 @@ update_properties (BluetoothSettingsWidget *self,
 	gtk_widget_hide (WID ("send_button"));
 
 	/* Name */
-	gtk_window_set_title (GTK_WINDOW (priv->properties_dialog), name);
+	gtk_window_set_title (GTK_WINDOW (priv->properties_dialog), alias);
 	g_free (priv->selected_name);
-	priv->selected_name = name;
+	priv->selected_name = alias;
 
 	/* Icon */
 	gtk_image_set_from_icon_name (GTK_IMAGE (WID ("image")), icon, GTK_ICON_SIZE_DIALOG);
@@ -1271,24 +1280,22 @@ static void
 update_visibility (BluetoothSettingsWidget *self)
 {
 	BluetoothSettingsWidgetPrivate *priv = BLUETOOTH_SETTINGS_WIDGET_GET_PRIVATE (self);
-	char *name;
+	g_autofree char *name = NULL;
 
 	g_object_get (G_OBJECT (priv->client), "default-adapter-name", &name, NULL);
 	if (name != NULL) {
-		char *label, *path, *uri;
+		g_autofree char *label = NULL;
+		g_autofree char *path = NULL;
+		g_autofree char *uri = NULL;
 
 		path = lookup_download_dir ();
 		uri = g_filename_to_uri (path, NULL, NULL);
-		g_free (path);
 
 		/* translators: first %s is the name of the computer, for example:
 		 * Visible as “Bastien Nocera’s Computer” followed by the
 		 * location of the Downloads folder.*/
 		label = g_strdup_printf (_("Visible as “%s” and available for Bluetooth file transfers. Transferred files are placed in the <a href=\"%s\">Downloads</a> folder."), name, uri);
-		g_free (uri);
-		g_free (name);
 		gtk_label_set_markup (GTK_LABEL (priv->visible_label), label);
-		g_free (label);
 	}
 	gtk_widget_set_visible (priv->visible_label, name != NULL);
 }
@@ -1334,7 +1341,7 @@ remove_selected_device (BluetoothSettingsWidget *self)
 {
 	BluetoothSettingsWidgetPrivate *priv = BLUETOOTH_SETTINGS_WIDGET_GET_PRIVATE (self);
 	GDBusProxy *adapter_proxy;
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 	GVariant *ret;
 
 	g_debug ("About to call RemoveDevice for %s", priv->selected_object_path);
@@ -1358,7 +1365,6 @@ remove_selected_device (BluetoothSettingsWidget *self)
 	if (ret == NULL) {
 		g_warning ("Failed to remove device '%s': %s",
 			   priv->selected_object_path, error->message);
-		g_error_free (error);
 	} else {
 		g_variant_unref (ret);
 	}
@@ -1406,20 +1412,19 @@ device_sort_func (gconstpointer a, gconstpointer b, gpointer data)
 	gboolean paired_a, paired_b;
 	gboolean trusted_a, trusted_b;
 	gboolean connected_a, connected_b;
-	char *name_a, *name_b;
-	int ret;
+	gint64 time_a, time_b;
 
 	g_object_get (row_a,
 		      "paired", &paired_a,
 		      "trusted", &trusted_a,
 		      "connected", &connected_a,
-		      "name", &name_a,
+		      "time-created", &time_a,
 		      NULL);
 	g_object_get (row_b,
 		      "paired", &paired_b,
 		      "trusted", &trusted_b,
 		      "connected", &connected_b,
-		      "name", &name_b,
+		      "time-created", &time_b,
 		      NULL);
 
 	/* First, paired or trusted devices (setup devices) */
@@ -1427,37 +1432,21 @@ device_sort_func (gconstpointer a, gconstpointer b, gpointer data)
 	setup_b = paired_b || trusted_b;
 	if (setup_a != setup_b) {
 		if (setup_a)
-			ret = -1;
+			return -1;
 		else
-			ret = 1;
-		goto out;
+			return 1;
 	}
 
 	/* Then connected ones */
 	if (connected_a != connected_b) {
 		if (connected_a)
-			ret = -1;
+			return -1;
 		else
-			ret = 1;
-		goto out;
+			return 1;
 	}
 
-	if (name_a == NULL) {
-		ret = 1;
-		goto out;
-	}
-	if (name_b == NULL) {
-		ret = -1;
-		goto out;
-	}
-
-	/* And all being equal, alphabetically */
-	ret = g_utf8_collate (name_a, name_b);
-
-out:
-	g_free (name_a);
-	g_free (name_b);
-	return ret;
+	/* And all being equal, oldest ones first */
+	return (time_a > time_b) ? 1 : -1;
 }
 
 static void
@@ -1557,7 +1546,10 @@ add_device_section (BluetoothSettingsWidget *self)
 	priv->device_label = gtk_label_new (s);
 	g_free (s);
 	gtk_label_set_use_markup (GTK_LABEL (priv->device_label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (priv->device_label), 0, 0.5);
+	g_object_set (G_OBJECT (priv->device_label),
+		      "xalign", 0.0,
+		      "yalign", 0.5,
+		      NULL);
 	gtk_widget_set_margin_end (priv->device_label, 6);
 	gtk_widget_set_margin_bottom (priv->device_label, 12);
 	gtk_box_pack_start (GTK_BOX (hbox), priv->device_label, FALSE, TRUE, 0);
@@ -1635,8 +1627,10 @@ row_inserted_cb (GtkTreeModel *tree_model,
 {
 	BluetoothSettingsWidget *self = user_data;
 	BluetoothSettingsWidgetPrivate *priv = BLUETOOTH_SETTINGS_WIDGET_GET_PRIVATE (user_data);
-	GDBusProxy *proxy;
-	char *name, *bdaddr;
+	g_autoptr(GDBusProxy) proxy = NULL;
+	g_autofree char *name = NULL;
+	g_autofree char *bdaddr = NULL;
+	g_autofree char *alias = NULL;
 	BluetoothType type;
 	gboolean paired, trusted, connected, legacy_pairing;
 	GtkWidget *row;
@@ -1646,13 +1640,13 @@ row_inserted_cb (GtkTreeModel *tree_model,
 				    BLUETOOTH_COLUMN_NAME, &name,
 				    -1);
 		g_debug ("Not adding device '%s'", name);
-		g_free (name);
 		return;
 	}
 
 	gtk_tree_model_get (tree_model, iter,
 			    BLUETOOTH_COLUMN_PROXY, &proxy,
 			    BLUETOOTH_COLUMN_NAME, &name,
+			    BLUETOOTH_COLUMN_ALIAS, &alias,
 			    BLUETOOTH_COLUMN_PAIRED, &paired,
 			    BLUETOOTH_COLUMN_TRUSTED, &trusted,
 			    BLUETOOTH_COLUMN_CONNECTED, &connected,
@@ -1672,18 +1666,16 @@ row_inserted_cb (GtkTreeModel *tree_model,
 			    "type", type,
 			    "connected", connected,
 			    "name", name,
+			    "alias", alias,
 			    "address", bdaddr,
 			    "legacy-pairing", legacy_pairing,
+			    "visible", name != NULL,
 			    NULL);
 	g_object_set_data_full (G_OBJECT (row), "object-path", g_strdup (g_dbus_proxy_get_object_path (proxy)), g_free);
 
 	gtk_container_add (GTK_CONTAINER (priv->device_list), row);
 	gtk_size_group_add_widget (priv->row_sizegroup, row);
 
-	g_object_unref (proxy);
-	g_free (name);
-	g_free (bdaddr);
-
 	gtk_stack_set_transition_type (GTK_STACK (priv->device_stack),
 				       GTK_STACK_TRANSITION_TYPE_SLIDE_DOWN);
 	gtk_container_child_set (GTK_CONTAINER (WID ("vbox_bluetooth")),
@@ -1726,12 +1718,14 @@ row_changed_cb (GtkTreeModel *tree_model,
 		path = g_object_get_data (G_OBJECT (l->data), "object-path");
 		if (g_str_equal (object_path, path)) {
 			g_autofree char *name = NULL;
+			g_autofree char *alias = NULL;
 			g_autofree char *bdaddr = NULL;
 			BluetoothType type;
 			gboolean paired, trusted, connected, legacy_pairing;
 
 			gtk_tree_model_get (tree_model, iter,
 					    BLUETOOTH_COLUMN_NAME, &name,
+					    BLUETOOTH_COLUMN_ALIAS, &alias,
 					    BLUETOOTH_COLUMN_PAIRED, &paired,
 					    BLUETOOTH_COLUMN_TRUSTED, &trusted,
 					    BLUETOOTH_COLUMN_CONNECTED, &connected,
@@ -1748,7 +1742,9 @@ row_changed_cb (GtkTreeModel *tree_model,
 				      "type", type,
 				      "connected", connected,
 				      "name", name,
+				      "alias", alias,
 				      "legacy-pairing", legacy_pairing,
+				      "visible", name != NULL,
 				      NULL);
 
 			/* Update the properties if necessary */
@@ -1776,11 +1772,10 @@ device_removed_cb (BluetoothClient *client,
 
 		path = g_object_get_data (G_OBJECT (l->data), "object-path");
 		if (g_str_equal (path, object_path)) {
-			char *name;
+			g_autofree char *name = NULL;
 
 			g_object_get (G_OBJECT (l->data), "name", &name, NULL);
-			g_debug ("Removing device '%s'", name);
-			g_free (name);
+			g_debug ("Removing device '%s'", name ? name : object_path);
 
 			gtk_widget_destroy (GTK_WIDGET (l->data));
 			found = TRUE;
@@ -1900,7 +1895,7 @@ static void
 setup_obex (BluetoothSettingsWidget *self)
 {
 	BluetoothSettingsWidgetPrivate *priv = BLUETOOTH_SETTINGS_WIDGET_GET_PRIVATE (self);
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 
 	priv->session_proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
 							     G_DBUS_PROXY_FLAGS_NONE,
@@ -1913,7 +1908,6 @@ setup_obex (BluetoothSettingsWidget *self)
 
 	if (priv->session_proxy == NULL) {
 		g_warning ("Failed to get session proxy: %s", error->message);
-		g_error_free (error);
 		return;
 	}
 	g_signal_connect (priv->session_proxy, "g-properties-changed",
@@ -1929,7 +1923,7 @@ bluetooth_settings_widget_init (BluetoothSettingsWidget *self)
 {
 	BluetoothSettingsWidgetPrivate *priv = BLUETOOTH_SETTINGS_WIDGET_GET_PRIVATE (self);
 	GtkWidget *widget;
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 
 	/* This ensures the BluetoothHdyColumn type is known by GtkBuilder when
 	 * loading the UI template.
@@ -1947,7 +1941,6 @@ bluetooth_settings_widget_init (BluetoothSettingsWidget *self)
                                        &error);
 	if (error != NULL) {
 		g_warning ("Could not load ui: %s", error->message);
-		g_error_free (error);
 		return;
 	}
 
